#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* 
	Sudoku solver by Paul Hsieh

    http://www.sudoku.org.uk/
    http://en.wikipedia.org/wiki/Sudoku

    Player:

    http://www.sudoku.com.au/

    Some hard/minimal sudoku puzzles:

    http://www.csse.uwa.edu.au/~gordon/sudokumin.php
 */

/*
	For each of the 81 cells, a boolean array is set for each of the 9 
	possible values that that cell might contain.  Each cell must have at 
	least one true value for that cell, and the sudoku is considered solved
    when each array has exactly one boolean value set to true.

    The board is initialized with some cells having only 1 truth set 
    (corresponging to the initial values given in the puzzle) while the rest
    have all boolean values set to true (corresponding to being empty.)

    The solver routines below successively start flipping many of the true
    values to false, corresponding to standard sudoku solving techniques.
    Only routines named searchDepth* will attempt any "trial an error" 
    techniques to solve the puzzle.

	The result is output in html format, as a solved or partially solved
    sudoku.  There are no known sudokus that this program has been unable to
    solve.  That said, the most general techniques (namely arbitrary depth 
    searching), which would guarantee a solution, have not been applied.
    Thus there is no *proof* that this program will solve all sudokus.
 */

/*
	The "boolean array" is just an int.  The bits, 1<<(1-1), through 1<<(9-1)
	are the boolean flags in question that correspond to the potential 
	occupancy of the numbers 1 through 9 respectively.
 */

int imask[9][9];
#define M(X) (1 << ((X)-1))

/*

	All the bits at once.

 */

#define MSK_ALL (M(1)|M(2)|M(3)|M(4)|M(5)|M(6)|M(7)|M(8)|M(9))

/*

	Read a puzzle.  The format is 9 lines of 9 characters each, occupied
	either by the numbers 1 through 9, or some other character to indicate a
	blank cell.

 */

int fetch (const char * fn) {
	FILE * fp = fopen (fn, "r");
	int i, j;

	if (NULL == fp) {
		fprintf (stderr, "Unable to read the file %s\n", fn);
		return -__LINE__;
	}

	for (i=0; i < 9; i++) {
		char s[1024];
		if (NULL == fgets (s, 1023, fp)) {
			fprintf (stderr, "Not enough lines.\nFile must contain 9 lines of 9 characters each\n");
			return -__LINE__;
		}
		if (strlen (s) < 9) {
			fprintf (stderr, "Not enough characters on line %d.\nFile must contain 9 lines of 9 characters each\n", i+1);
			return -__LINE__;
		}
		for (j=0; j < 9; j++) {
			if (s[j] < '1' || s[j] > '9') imask[i][j] = MSK_ALL;
			else imask[i][j] = M(s[j] - '0');
		}
	}
	fclose (fp);
	return 0;
}

/*
	Test to see if a cell is composed of a single set flag or not.  Return the
	flag itself if it is, otherwise return 0.
*/

#define issingletonRaw(x) (((x) & ((x)-1)) == 0)

unsigned int singletontable[512];
#define singleton(x) singletontable[(x)]
#define issingleton(x) singleton(x)

/*
	Test to see if the sudoku is solved.
*/

int issolved (void) {
	int i, j;
	for (i=0; i < 9; i++) for (j=0; j < 9; j++) {
		if (imask[i][j] == 0 || !issingleton(imask[i][j])) return 0;
	}
	return 1;
}

/*
	Apply the most obvious first form of logic.  I.e., for any singleton, 
	every other boolean array intersecting that same column, row, or 3x3
	grid may not have that boolean flag set to true.
*/

#define srow3(i,j)  (singleton(imask[i][j])|singleton(imask[i][j+1])|singleton(imask[i][j+2]))
#define scol3(i,j)  (singleton(imask[i][j])|singleton(imask[i+1][j])|singleton(imask[i+2][j]))

#define srow(i)     (srow3(i,0)|srow3(i,3)|srow3(i,6))
#define scol(j)     (scol3(0,j)|scol3(3,j)|scol3(6,j))
#define sblock(i,j) (srow3(i,j)|srow3(i+1,j)|srow3(i+2,j))

void simpleLogic (int i, int j) {
	int s0 = sblock (i, j);
	int s1, s2[3];
	int s, x, y;

	s2[0] = scol(j);
	s2[1] = scol(j+1);
	s2[2] = scol(j+2);

	for (x=i; x < i+3; x++) {
		s1 = srow (x);
		for (y=j; y < j+3; y++) {
			if (!issingleton (imask[x][y])) {
				s = MSK_ALL ^ (s0 | s1 | s2[y-j]);
				imask[x][y] &= s;
			}
		}
	}
}

void simpleLogicPass1 (void) {
	simpleLogic (0, 0);
	simpleLogic (0, 3);
	simpleLogic (0, 6);
	simpleLogic (3, 0);
	simpleLogic (3, 3);
	simpleLogic (3, 6);
	simpleLogic (6, 0);
	simpleLogic (6, 3);
	simpleLogic (6, 6);
}

/*
	The next logic technique used to try to solve is the idea of disjoint
	groups.  For example, if two "intersecting" cells both contain exactly 
	the same boolean arrays that themselves contain exactly two entries, then 
	no other cells that intersect the two may have either of those two flags
    set.
*/

unsigned char occupancymaskcount[512];
#define occupancycount(x) ((int) occupancymaskcount[(x)])

#define DSB_INNER(a,b) if ((m & imask[a][b]) == imask[a][b]) c--;

#define disjointSetsBlock(i,j) { \
	for (x=i; x < i+3; x++) for (y=j; y < j+3; y++) { \
		if (1 < (c = occupancycount (m = imask[x][y]))) { \
			DSB_INNER(i+0,j); DSB_INNER(i+0,j+1); DSB_INNER(i+0,j+2); \
			DSB_INNER(i+1,j); DSB_INNER(i+1,j+1); DSB_INNER(i+1,j+2); \
			DSB_INNER(i+2,j); DSB_INNER(i+2,j+1); DSB_INNER(i+2,j+2); \
			if (c <= 0) { \
				for (a=i; a < i+3; a++) for (b=j; b < j+3; b++) { \
					if ((m & imask[a][b]) != imask[a][b]) { \
						imask[a][b] &= ~m; \
					} \
				} \
			} \
		} \
	} \
}

void disjointSetsBlocks (void) {
int a, b, c, x, y, m;
	disjointSetsBlock (0, 0);
	disjointSetsBlock (0, 3);
	disjointSetsBlock (0, 6);
	disjointSetsBlock (3, 0);
	disjointSetsBlock (3, 3);
	disjointSetsBlock (3, 6);
	disjointSetsBlock (6, 0);
	disjointSetsBlock (6, 3);
	disjointSetsBlock (6, 6);
}

#define DSRINNER(k) if ((m & imask[i][k]) == imask[i][k]) c--;

void disjointSetsRow (int i) {
int c, m, j, k;

	for (j=0; j < 9; j++) {
		if (1 < (c = occupancycount (m = imask[i][j]))) {

			DSRINNER(0); DSRINNER(1); DSRINNER(2);
			DSRINNER(3); DSRINNER(4); DSRINNER(5);
			DSRINNER(6); DSRINNER(7); DSRINNER(8);

			if (c <= 0) {
				for (k=0; k < 9; k++) {
					if ((m & imask[i][k]) != imask[i][k]) {
						imask[i][k] &= ~m;
					}
				}
			}
		}
	}
}

void disjointSetsRows (void) {
	disjointSetsRow (0);
	disjointSetsRow (1);
	disjointSetsRow (2);
	disjointSetsRow (3);
	disjointSetsRow (4);
	disjointSetsRow (5);
	disjointSetsRow (6);
	disjointSetsRow (7);
	disjointSetsRow (8);
}

#define DSCINNER(k) if ((m & imask[k][j]) == imask[k][j]) c--;

void disjointSetsCol (int j) {
int c, m, i, k;

	for (i=0; i < 9; i++) {
		if (1 < (c = occupancycount (m = imask[i][j]))) {
			DSCINNER(0); DSCINNER(1); DSCINNER(2);
			DSCINNER(3); DSCINNER(4); DSCINNER(5);
			DSCINNER(6); DSCINNER(7); DSCINNER(8);

			if (c <= 0) {
				for (k=0; k < 9; k++) {
					if ((m & imask[k][j]) != imask[k][j]) {
						imask[k][j] &= ~m;
					}
				}
			}
		}
	}
}

void disjointSetsCols (void) {
	disjointSetsCol (0);
	disjointSetsCol (1);
	disjointSetsCol (2);
	disjointSetsCol (3);
	disjointSetsCol (4);
	disjointSetsCol (5);
	disjointSetsCol (6);
	disjointSetsCol (7);
	disjointSetsCol (8);
}

/*
	The next logic technique just examines, for each value, if there is only 
	one cell in an intersection structure (row, column or 3x3 grid) which has 
	its boolean flag set to true for that value, then it must uniquely be 
	that value.
*/

#define UBINNER(x,y) if (v & imask[x][y]) { \
	c++; \
	if (c > 1) continue; \
	a = x; \
	b = y; \
}

#define uniqueBlock(i,j) { \
	for (v = M(1); v <= M(9); v += v) { \
		c = 0; \
		UBINNER(i+0,j); UBINNER(i+0,j+1); UBINNER(i+0,j+2); \
		UBINNER(i+1,j); UBINNER(i+1,j+1); UBINNER(i+1,j+2); \
		UBINNER(i+2,j); UBINNER(i+2,j+1); UBINNER(i+2,j+2); \
		if (1 == c) imask[a][b] = v; \
	} \
}

void uniqueBlocks (void) {
int a, b, c, v;

	uniqueBlock (0, 0);
	uniqueBlock (0, 3);
	uniqueBlock (0, 6);
	uniqueBlock (3, 0);
	uniqueBlock (3, 3);
	uniqueBlock (3, 6);
	uniqueBlock (6, 0);
	uniqueBlock (6, 3);
	uniqueBlock (6, 6);
}

#define URINNER(j) if (v & imask[i][j]) { \
	c++; \
	if (c > 1) goto Next; \
	k = j; \
}

void uniqueRow (int i) {
int c, k, v;

	for (v = M(1); v <= M(9); v += v) {
		c = 0;

		URINNER(0); URINNER(1); URINNER(2);
		URINNER(3); URINNER(4); URINNER(5);
		URINNER(6); URINNER(7); URINNER(8);

		if (1 == c) imask[i][k] = v;
		Next:;
	}
}

void uniqueRows (void) {
	uniqueRow (0);
	uniqueRow (1);
	uniqueRow (2);
	uniqueRow (3);
	uniqueRow (4);
	uniqueRow (5);
	uniqueRow (6);
	uniqueRow (7);
	uniqueRow (8);
}

#define UCINNER(i) if (v & imask[i][j]) { \
	c++; \
	if (c > 1) goto Next; \
	k = i; \
}

void uniqueCol (int j) {
int c, k, v;

	for (v = M(1); v <= M(9); v += v) {
		c = 0;

		UCINNER(0); UCINNER(1); UCINNER(2);
		UCINNER(3); UCINNER(4); UCINNER(5);
		UCINNER(6); UCINNER(7); UCINNER(8);

		if (1 == c) imask[k][j] = v;
		Next:;
	}
}

void uniqueCols (void) {
	uniqueCol (0);
	uniqueCol (1);
	uniqueCol (2);
	uniqueCol (3);
	uniqueCol (4);
	uniqueCol (5);
	uniqueCol (6);
	uniqueCol (7);
	uniqueCol (8);
}

/*
	Generalization of "simple" logic pass.  If a value appears in only one
	row or column of a block, then cannot appear in the intersecting rows or
	columns of other blocks.
*/

void blockIsolateCols (int i, int j) {
int v, x, y, z, c[3];

	for (x=0; x < 3; x++) c[x] = imask[i][j+x]|imask[i+1][j+x]|imask[i+2][j+x];

	if (MSK_ALL != (v = c[1]|c[2])) for (y=i,z=0; z < 2; z++) {
		y += 3; if (y >= 9) y = 0;
		imask[y+0][j] &= v;
		imask[y+1][j] &= v;
		imask[y+2][j] &= v;
	}

	if (MSK_ALL != (v = c[0]|c[2])) for (y=i,z=0; z < 2; z++) {
		y += 3; if (y >= 9) y = 0;
		imask[y+0][j+1] &= v;
		imask[y+1][j+1] &= v;
		imask[y+2][j+1] &= v;
	}

	if (MSK_ALL != (v = c[0]|c[1])) for (y=i,z=0; z < 2; z++) {
		y += 3; if (y >= 9) y = 0;
		imask[y+0][j+2] &= v;
		imask[y+1][j+2] &= v;
		imask[y+2][j+2] &= v;
	}
}

void blockIsolateRows (int i, int j) {
int v, x, y, z, c[3];

	for (x=0; x < 3; x++) c[x] = imask[i+x][j]|imask[i+x][j+1]|imask[i+x][j+2];

	if (MSK_ALL != (v = c[1]|c[2])) for (y=j,z=0; z < 2; z++) {
		y += 3; if (y >= 9) y = 0;
		imask[i][y+0] &= v;
		imask[i][y+1] &= v;
		imask[i][y+2] &= v;
	}

	if (MSK_ALL != (v = c[0]|c[2])) for (y=j,z=0; z < 2; z++) {
		y += 3; if (y >= 9) y = 0;
		imask[i+1][y+0] &= v;
		imask[i+1][y+1] &= v;
		imask[i+1][y+2] &= v;
	}

	if (MSK_ALL != (v = c[0]|c[1])) for (y=j,z=0; z < 2; z++) {
		y += 3; if (y >= 9) y = 0;
		imask[i+2][y+0] &= v;
		imask[i+2][y+1] &= v;
		imask[i+2][y+2] &= v;
	}
}

void blockIsolate (int i, int j) {
	blockIsolateCols (i, j);
	blockIsolateRows (i, j);
}

void blockIsolateAll (void) {
	blockIsolate (0, 0);
	blockIsolate (0, 3);
	blockIsolate (0, 6);
	blockIsolate (3, 0);
	blockIsolate (3, 3);
	blockIsolate (3, 6);
	blockIsolate (6, 0);
	blockIsolate (6, 3);
	blockIsolate (6, 6);
}

/*
	Iteratively apply the logical deductions above until they stop yielding
	more simplifications.  Return true if the puzzle is solved, otherwise
	false.
*/

int scount = 0;

int shallowDeductions (void) {
	int j;
	static int store_imask[9][9];

	scount ++;

	for (j=0; /* j < (81*8) */; j++) {
		memcpy (store_imask, imask, sizeof (imask));

		simpleLogicPass1 ();
		blockIsolateAll ();

		if (0 != memcmp (store_imask, imask, sizeof (imask))) continue;

		disjointSetsBlocks ();
		disjointSetsRows ();
		disjointSetsCols ();

		if (0 != memcmp (store_imask, imask, sizeof (imask))) continue;

		uniqueBlocks ();
		uniqueRows ();
		uniqueCols ();

		if (0 == memcmp (store_imask, imask, sizeof (imask))) {
			/* fprintf (stderr, "%d iterations\n", j); */
			break;
		}
	}

	return issolved ();
}

/*
	Check for validity of a given puzzle in its current state.
*/

#define chkrow3(i,j)  (imask[i][j]|imask[i][j+1]|imask[i][j+2])
#define chkcol3(i,j)  (imask[i][j]|imask[i+1][j]|imask[i+2][j])

#define chkrow(i)     (chkrow3(i,0)|chkrow3(i,3)|chkrow3(i,6))
#define chkcol(j)     (chkcol3(0,j)|chkcol3(3,j)|chkcol3(6,j))
#define chkblock(i,j) (chkrow3(i,j)|chkrow3(i+1,j)|chkrow3(i+2,j))

static int checkValidGrid (void) {
	int i, j;

	for (i=0; i < 9; i++) if (chkrow(i) != MSK_ALL) return 0;
	for (j=0; j < 9; j++) if (chkcol(j) != MSK_ALL) return 0;
	for (i=0; i < 9; i+=3) for (j=0; j < 9; j+=3) if (chkblock(i,j) != MSK_ALL) return 0;

	return 1;
}

/*
	Solve by single "guessing" in every cell that doesn't have a unique
	value.  If the guess yields a unique solution, it is checked against
	all other possible guesses to make sure that other solutions are not
	possible.  I.e., the puzzle is considered flawed if there is more than
	one solution.  If the guess yields an improper solution or does not
	solve the puzzle, then it is thrown out.
*/

int deep = 0, potential = 0;

int searchDepth1 (int i, int j) {
	static int store_imask[9][9];
	static int solution_imask[9][9];
	static int prune_imask[9][9];
	int k, f = 0;

	memset (prune_imask, 0, sizeof (prune_imask));
	memcpy (store_imask, imask, sizeof (imask));
	for (; i < 9; i++, j = 0) for (; j < 9; j++) if (!singleton (imask[i][j])) {
		for (k=M(1); k <= M(9); k += k) if ((store_imask[i][j] & k) && !(prune_imask[i][j] & k)) {
			imask[i][j] = k;
			if (shallowDeductions ()) {
				if (1 == f) {
					if (0 != memcmp (solution_imask, imask, sizeof (imask))) {
						if (checkValidGrid ()) return -1; /* Duplicate */
					}
				} else if (checkValidGrid ()) {
					memcpy (solution_imask, imask, sizeof (imask));
					f = 1;
					if (0 == potential) {
						fprintf (stderr, "At least one possible solution found\n");
						potential = 1;
					}
					goto Prune;
				}
			} else {
				if (checkValidGrid ()) {
					int ii, jj;
					Prune:;
					for (ii = i, jj = j+1; ii < 9; ii++, jj=0) for (;jj < 9; jj++) {
						if (!singleton (store_imask[ii][jj])) {
							if (singleton (imask[ii][jj])) {
								prune_imask[ii][jj] |= imask[ii][jj];
							}
						}
					}
				}
			}
			memcpy (imask, store_imask, sizeof (imask));
		}
	}
	if (1 == f) memcpy (imask, solution_imask, sizeof (imask));
	return f;
}

/*
	Perform a single recursion on top of the single guess above.  I.e., so
	puzzles that require no more than two "pending guesses" can be solved.
*/

int searchDepth2 (int i, int j) {
	static int store_imask[9][9];
	static int solution_imask[9][9];
	static int prune_imask[9][9];
	int k, f = 0;

	if (0 != (k = searchDepth1 (i, j))) return k;

	if (0 == deep) {
		fprintf (stderr, "A deep search is required\n");
		deep = 1;
	}

	memset (prune_imask, 0, sizeof (prune_imask));
	memcpy (store_imask, imask, sizeof (imask));
	for (; i < 9; i++, j = 0) for (; j < 9; j++) if (!singleton (imask[i][j])) {
		for (k=M(1); k <= M(9); k += k) if ((store_imask[i][j] & k) && !(prune_imask[i][j] & k)) {
			imask[i][j] = k;
			shallowDeductions ();
			if (checkValidGrid ()) {
				int ii, jj, rv;
				if (1 == (rv = searchDepth1 (i, j))) {
					if (1 == f) {
						if (0 != memcmp (solution_imask, imask, sizeof (imask))) {
							if (checkValidGrid ()) return -1; /* Duplicate */
						}
					} else if (checkValidGrid ()) {
						memcpy (solution_imask, imask, sizeof (imask));
						f = 1;
					}
				}

				/* If an (i,j) guess implies a later (ii,jj) must be unique then an
				   (ii,jj) guess cannot imply any more than the (i,j) guess.  Thus
				   we can prune any such (ii,jj) from future consideration. */

				if (rv >= 0) for (ii = i, jj = j+1; ii < 9; ii++, jj = 0) for (;jj < 9; jj++) {
					if (!singleton (store_imask[ii][jj]) &&
						singleton (imask[ii][jj])) {
						prune_imask[ii][jj] |= imask[ii][jj];
					}
				}
			}
			memcpy (imask, store_imask, sizeof (imask));
		}
	}
	if (1 == f) memcpy (imask, solution_imask, sizeof (imask));
	return f;
}

/*
	Output the result as HTML 4.0 with CSS.
 */

void output (void) {
int i, j, k;
int x, y;

	printf ("<!doctype html public \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n");
	printf ("<html>\n");

	printf ("<style type=\"text/css\">\n");
	printf ("  body {\n");
	printf ("    font: 32px Arial, sans-serif, Lucida Grande, Verdana, Helvetica;\n");
	printf ("  }\n");
	printf ("  table {\n");
	printf ("    font: 32px Arial, sans-serif, Lucida Grande, Verdana, Helvetica;\n");
	printf ("    background: #000000;\n");
	printf ("  }\n");
	printf ("  table.white {\n");
	printf ("    background: white;\n");
	printf ("  }\n");
	printf ("  table.solidBlackBorder {\n");
	printf ("    font: 14px Arial, sans-serif, Lucida Grande, Verdana, Helvetica;\n");
	printf ("    background: #000000;\n");
	printf ("    text-align: center;\n");
	printf ("  }\n");
	printf ("  td {\n");
	printf ("    background: white;\n");
	printf ("    text-align: center;\n");
	printf ("  }\n");
	printf ("  td.black {\n");
	printf ("    background: black;\n");
	printf ("  }\n");
	printf ("</style>\n");

	printf ("<body><table>\n");
	for (x=0; x < 9; x+=3) {
		printf (" <tr height=\"96\">\n");
		for (y=0; y < 9; y+=3) {
			printf ("  <td class=\"black\" width=\"114\">\n");
			printf ("   <table>\n");
			for (i=x; i < x+3; i++) {
				printf ("    <tr height=\"32\">\n");
				for (j=y; j < y+3; j++) {
					printf ("     <td width=\"38\">\n");

					if (imask[i][j] && issingleton (imask[i][j])) {
						for (k=1; k <= 9; k++) {
							if (imask[i][j] & M(k)) printf ("      %d\n", k);
						}
					} else {
						int f;
						if (imask[i][j] == MSK_ALL) {
							printf ("   *\n");
						} else if (0 == (MSK_ALL & imask[i][j])) {
							printf ("   ?\n");
						} else {
							if (!singleton (imask[i][j])) printf ("<font size = -1>");
							printf ("   <em>");
							for (f = 0, k = 1; k <= 9; k++) {
								if (imask[i][j] & M(k)) {
									if (f) {
										if (0 == (f % 2)) printf ("<br>");
										else printf ("&nbsp;");
									}
									printf ("%d", k);
									f++;
								}
							}
							printf ("</em>\n");
							if (!singleton (imask[i][j])) printf ("</font>");
						}
					}

					printf ("     </td>\n");
				}
				printf ("    </tr>\n");
			}
			printf ("   </table>\n");
			printf ("  </td>\n");
		}
		printf (" </tr>\n");
	}
	printf ("</table></body></html>\n");
}

int main (int argc, char * argv[]) {
clock_t t0, t1;

	if (argc < 2) {
		fprintf (stderr, "%s <filename>\n", argv[0]);
		return -__LINE__;
	}
	if (0 > fetch (argv[1])) {
		fprintf (stderr, "bad format in %s\n", argv[1]);
		return -__LINE__;
	}

	if (!checkValidGrid ()) {
		fprintf (stderr, "Invalid starting state.  (There is a repeat in the source data.)\n");
		return -__LINE__;
	}

	t0 = clock ();

	{
		int i, j, x;
		for (j=0; j < 512; j++) {
			for (i=0, x=j; x; x &= x-1) i++;
			occupancymaskcount[j] = i;
			singletontable[j] = issingletonRaw (j) ? j : 0;
		}
	}

	if (!shallowDeductions ()) {
		int ret;

		if (!checkValidGrid ()) {
			fprintf (stderr, "Not solveable.\n");
		} else {
			fprintf (stderr, "Search required\n");
			if (1 != (ret = searchDepth2 (0, 0))) {
				if (ret < 0) {
					fprintf (stderr, "Multiple solutions\n");
				} else {
					if (!checkValidGrid ()) {
						fprintf (stderr, "Not solveable.\n");
					} else {
						fprintf (stderr, "Not solved\n");
					}
				}
			}
		}
	}

	t1 = clock ();

	fprintf (stderr, "Time: %f\n", (t1 - t0)/((double) CLOCKS_PER_SEC));
	/* fprintf (stderr, "Total node count: %d\n", scount); */

	output ();
	return 0;
}
